package com.bartoszwalter.students.bank.model;

import java.util.ArrayList;
import java.util.List;

import com.bartoszwalter.students.bank.BankingException;
import com.bartoszwalter.students.bank.operation.Operation;
import com.bartoszwalter.students.bank.report.Report;

public class NonDebitAccount implements BankingProduct, Account {
	private String id;
	private String givenName, familyName;
	private int balance;
	private List history = new ArrayList();
	private char interestType;
	
	/**
	 * Utworzenie rachunku
	 * @param id
	 * @param givenName
	 * @param familyName
	 */
	public NonDebitAccount(String id, String givenName, String familyName) {
		this.id = id;
		this.givenName = givenName;
		this.familyName = familyName;
		this.balance = 0;
		interestType = 'A';
	}
	
	/* (non-Javadoc)
	 * @see com.bartoszwalter.students.bank.model.Account#id()
	 */
	@Override
	public String id() {
		return id;
	}
	
	/* (non-Javadoc)
	 * @see com.bartoszwalter.students.bank.model.Account#owner()
	 */
	@Override
	public String owner() {
		return givenName + " " + familyName;
	}
	
	/* (non-Javadoc)
	 * @see com.bartoszwalter.students.bank.model.Account#balance()
	 */
	@Override
	public int balance() {
		return balance;
	}	

	/* (non-Javadoc)
	 * @see com.bartoszwalter.students.bank.model.Account#printHistory()
	 */
	@Override
	public void printHistory() {
		System.out.println(history);
	}
	
	/* (non-Javadoc)
	 * @see com.bartoszwalter.students.bank.model.Account#credit(int)
	 */
	@Override
	public void credit(int kwota) {
		balance += kwota;
	}
	
	/* (non-Javadoc)
	 * @see com.bartoszwalter.students.bank.model.Account#debit(int)
	 */
	@Override
	public void debit(int kwota) throws BankingException {
		if (balance < kwota) 
			throw new BankingException("Insufficient funds on account " + id());
		
		balance -= kwota;
	}
	
	/* (non-Javadoc)
	 * @see com.bartoszwalter.students.bank.model.Account#calculateInterest()
	 */
	@Override
	public int calculateInterest() {
		int value = 0;
		if (interestType == 'A') {
			if (balance < 1000)
				value = (int) (0.1 * balance);
			if (balance < 3000)
				value = (int) (0.1 * 1000 + 0.2 * (balance - 1000));
			else
				value = (int) (0.1 * 1000 + 0.2 * 2000 + 0.3 * (balance - 3000));
		} else if (interestType == 'B') {
			if (balance < 2000)
				value = (int) (0.2 * balance);
			if (balance < 10000)
				value = (int) (0.2 * 2000 + 0.4 * (balance - 10000));
			else
				value = (int) (0.2 * 2000 + 0.4 * 10000 + 0.5 * (balance - 20000));
		} else if (interestType == 'C') {
			value = (int) (0.3 * balance);
		}
		
		history.add("Naliczono odsetki w kwocie " + value);
		
		return value;
	}

	/* (non-Javadoc)
	 * @see com.bartoszwalter.students.bank.model.Account#accept(com.bartoszwalter.students.bank.report.Report)
	 */
	@Override
	public boolean accept(Report report) {
		if (report.isPriviliged())
			return false;
		report.visit(this);
		return true;
	}

	/* (non-Javadoc)
	 * @see com.bartoszwalter.students.bank.model.Account#addHistory(com.bartoszwalter.students.bank.operation.Operation)
	 */
	@Override
	public void addHistory(Operation operation) {
		history.add(operation);		
	}
}
